//=============================================================================
//
// file :        A2605.h
//
// description : Include for the A2605 class.
//
// project :	A2605-A2620 Power Supply device server
//
// $Author: giulio $
//
// $Revision: 1.12 $
//
// $Log: A2605.h,v $
// Revision 1.12  2019-04-15 06:28:40  giulio
// Moved to tango9
//
// Revision 1.11  2013-08-23 10:24:44  giulio
// Moved to Device4Impl
//
// Revision 1.10  2013-04-16 09:44:23  giulio
// Update rt-buffer api's, removed tango rpath, added DeviceState
//
// Revision 1.9  2012-04-19 09:28:22  giulio
// Added command logging buffer
//
// Revision 1.8  2012-01-02 12:55:29  giulio
// .Changed logic in local+rnm current control, add degauss, updated cycling procedure
//
// Revision 1.7  2011/01/21 13:15:38  giulio
// .Fixed save property issue
//
// Revision 1.6  2011/01/17 13:30:20  giulio
// .Set and save ps mac address
//
// Revision 1.5  2011/01/17 11:12:44  giulio
// .Off managed by thread
//
// Revision 1.4  2010/09/27 12:01:42  giulio
// .Check set/read difference.Fault state when difference above a threshold for a certain amount of time
//
// Revision 1.3  2010/09/20 06:55:57  giulio
// .Added support for non realtime communication protocol
//
// Revision 1.2  2010/08/26 13:11:24  giulio
// Fixed a bug in get_wave api
//
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _A2605_H
#define _A2605_H

#include <tango.h>

#include <buffer.h>

#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <stdio.h>


#include <a2605.h>

#define A2605_CURRSET_IDX    0
#define A2605_CURRREAD_IDX  1
#define A2605_VOLTAGE_IDX     2
#define A2605_VDCLINK_IDX      3

#define A2605_REBOOT_DSP_PORT 30704
#define A2605_MSG_LEN 50

#define A2605_STAT_TOUT     1000000    /* sec */
#define A2605_SOCK_TOUT    10  /* sec */

//using namespace Tango;

/**
 * @author	$Author: giulio $
 * @version	$Revision: 1.12 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------


namespace A2605_ns
{
	class cyclethread;
	class offthread;
/**
 * Class Description:
 * Device server for the A2605 +-5A/20A power supply developed inhouse.
 */

/*
 *	Device States Description:
*  Tango::ON :
*  Tango::OFF :
*  Tango::FAULT :
 */


class A2605: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevDouble	*attr_Current_read;
		Tango::DevDouble	attr_Current_write;
		Tango::DevDouble	*attr_CurrentFb_read;
		Tango::DevDouble	*attr_VdcLink_read;
		Tango::DevDouble	*attr_Voltage_read;
		Tango::DevBoolean	*attr_WaveAbsCurMode_read;
		Tango::DevBoolean	attr_WaveAbsCurMode_write;
		Tango::DevLong	*attr_WaveBunchNumEnd_read;
		Tango::DevLong	*attr_WaveBunchNumStart_read;
		Tango::DevLong	attr_WaveBunchNumStart_write;
		Tango::DevLong	*attr_WaveCycles_read;
		Tango::DevLong	attr_WaveCycles_write;
		Tango::DevBoolean	*attr_WaveLoopMode_read;
		Tango::DevBoolean	attr_WaveLoopMode_write;
		Tango::DevLong	*attr_WaveStepFactor_read;
		Tango::DevLong	attr_WaveStepFactor_write;
		Tango::DevBoolean	*attr_WaveTrigBunchNum_read;
		Tango::DevBoolean	attr_WaveTrigBunchNum_write;
		Tango::DevBoolean	*attr_WaveTrigIrq_read;
		Tango::DevBoolean	attr_WaveTrigIrq_write;
		Tango::DevBoolean	*attr_WaveTrigSw_read;
		Tango::DevBoolean	attr_WaveTrigSw_write;
		Tango::DevShort	*attr_CurrentControl_read;
		Tango::DevShort	attr_CurrentControl_write;
		Tango::DevLong	*attr_BunchNumber_read;
		Tango::DevDouble	*attr_CurrentSet_read;
		Tango::DevDouble	attr_CurrentSet_write;
		Tango::DevDouble	*attr_CommFrequency_read;
		Tango::DevString	*attr_PsMacAddress_read;
		Tango::DevString	attr_PsMacAddress_write;
		Tango::DevUShort	*attr_DeviceState_read;
		Tango::DevUShort	attr_DeviceState_write;
		Tango::DevBoolean	*attr_PsStat_read;
		Tango::DevDouble	*attr_Waveform_read;
		Tango::DevDouble	attr_Waveform_write;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	Power supply ip number
 */
	string	psIp;
/**
 *	Power supply mac address
 */
	string	psMacAddress;
/**
 *	Power supply udp port
 */
	Tango::DevShort	psUdpPort;
/**
 *	Cpu udp port
 */
	Tango::DevShort	cpuUdpPort;
/**
 *	Power supply model (5/20)
 */
	Tango::DevShort	psModel;
/**
 *	Power supply index
 */
	Tango::DevLong	psIndex;
/**
 *	Rnm index
 */
	Tango::DevLong	rnmIndex;
/**
 *	
 */
	string	cpuIp;
/**
 *	If 0 disable slew rate control.
 */
	Tango::DevDouble	slewRateLimit;
/**
 *	0=local control
 *	1=feedback (remote) control
 *	2=local+feedback control
 */
	Tango::DevShort	currentControl;
/**
 *	Realtime communication
 */
	Tango::DevBoolean	realtimeControl;
/**
 *	If set-read differ for more then DiffTimeErr, then stat in fault
 */
	Tango::DevLong	diffTimeErr;
/**
 *	If current differs for more then this current (A), then device in fault.
 */
	Tango::DevDouble	diffCurrErr;
/**
 *	Minimum delta current(set-read) under which ramp is disabled
 */
	Tango::DevDouble	minDeltaCurRamp;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	A2605(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	A2605(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	A2605(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~A2605() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name A2605 methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Current acquisition result.
 */
	virtual void read_Current(Tango::Attribute &attr);
/**
 *	Write Current attribute values to hardware.
 */
	virtual void write_Current(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CurrentFb acquisition result.
 */
	virtual void read_CurrentFb(Tango::Attribute &attr);
/**
 *	Extract real attribute values for VdcLink acquisition result.
 */
	virtual void read_VdcLink(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Voltage acquisition result.
 */
	virtual void read_Voltage(Tango::Attribute &attr);
/**
 *	Extract real attribute values for WaveAbsCurMode acquisition result.
 */
	virtual void read_WaveAbsCurMode(Tango::Attribute &attr);
/**
 *	Write WaveAbsCurMode attribute values to hardware.
 */
	virtual void write_WaveAbsCurMode(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveBunchNumEnd acquisition result.
 */
	virtual void read_WaveBunchNumEnd(Tango::Attribute &attr);
/**
 *	Extract real attribute values for WaveBunchNumStart acquisition result.
 */
	virtual void read_WaveBunchNumStart(Tango::Attribute &attr);
/**
 *	Write WaveBunchNumStart attribute values to hardware.
 */
	virtual void write_WaveBunchNumStart(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveCycles acquisition result.
 */
	virtual void read_WaveCycles(Tango::Attribute &attr);
/**
 *	Write WaveCycles attribute values to hardware.
 */
	virtual void write_WaveCycles(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveLoopMode acquisition result.
 */
	virtual void read_WaveLoopMode(Tango::Attribute &attr);
/**
 *	Write WaveLoopMode attribute values to hardware.
 */
	virtual void write_WaveLoopMode(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveStepFactor acquisition result.
 */
	virtual void read_WaveStepFactor(Tango::Attribute &attr);
/**
 *	Write WaveStepFactor attribute values to hardware.
 */
	virtual void write_WaveStepFactor(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveTrigBunchNum acquisition result.
 */
	virtual void read_WaveTrigBunchNum(Tango::Attribute &attr);
/**
 *	Write WaveTrigBunchNum attribute values to hardware.
 */
	virtual void write_WaveTrigBunchNum(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveTrigIrq acquisition result.
 */
	virtual void read_WaveTrigIrq(Tango::Attribute &attr);
/**
 *	Write WaveTrigIrq attribute values to hardware.
 */
	virtual void write_WaveTrigIrq(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for WaveTrigSw acquisition result.
 */
	virtual void read_WaveTrigSw(Tango::Attribute &attr);
/**
 *	Write WaveTrigSw attribute values to hardware.
 */
	virtual void write_WaveTrigSw(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CurrentControl acquisition result.
 */
	virtual void read_CurrentControl(Tango::Attribute &attr);
/**
 *	Write CurrentControl attribute values to hardware.
 */
	virtual void write_CurrentControl(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for BunchNumber acquisition result.
 */
	virtual void read_BunchNumber(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CurrentSet acquisition result.
 */
	virtual void read_CurrentSet(Tango::Attribute &attr);
/**
 *	Write CurrentSet attribute values to hardware.
 */
	virtual void write_CurrentSet(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CommFrequency acquisition result.
 */
	virtual void read_CommFrequency(Tango::Attribute &attr);
/**
 *	Extract real attribute values for PsMacAddress acquisition result.
 */
	virtual void read_PsMacAddress(Tango::Attribute &attr);
/**
 *	Write PsMacAddress attribute values to hardware.
 */
	virtual void write_PsMacAddress(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for DeviceState acquisition result.
 */
	virtual void read_DeviceState(Tango::Attribute &attr);
/**
 *	Write DeviceState attribute values to hardware.
 */
	virtual void write_DeviceState(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for PsStat acquisition result.
 */
	virtual void read_PsStat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Waveform acquisition result.
 */
	virtual void read_Waveform(Tango::Attribute &attr);
/**
 *	Write Waveform attribute values to hardware.
 */
	virtual void write_Waveform(Tango::WAttribute &attr);
/**
 *	Read/Write allowed for Current attribute.
 */
	virtual bool is_Current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CurrentFb attribute.
 */
	virtual bool is_CurrentFb_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for VdcLink attribute.
 */
	virtual bool is_VdcLink_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Voltage attribute.
 */
	virtual bool is_Voltage_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveAbsCurMode attribute.
 */
	virtual bool is_WaveAbsCurMode_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveBunchNumEnd attribute.
 */
	virtual bool is_WaveBunchNumEnd_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveBunchNumStart attribute.
 */
	virtual bool is_WaveBunchNumStart_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveCycles attribute.
 */
	virtual bool is_WaveCycles_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveLoopMode attribute.
 */
	virtual bool is_WaveLoopMode_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveStepFactor attribute.
 */
	virtual bool is_WaveStepFactor_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveTrigBunchNum attribute.
 */
	virtual bool is_WaveTrigBunchNum_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveTrigIrq attribute.
 */
	virtual bool is_WaveTrigIrq_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for WaveTrigSw attribute.
 */
	virtual bool is_WaveTrigSw_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CurrentControl attribute.
 */
	virtual bool is_CurrentControl_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BunchNumber attribute.
 */
	virtual bool is_BunchNumber_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CurrentSet attribute.
 */
	virtual bool is_CurrentSet_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CommFrequency attribute.
 */
	virtual bool is_CommFrequency_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for PsMacAddress attribute.
 */
	virtual bool is_PsMacAddress_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for DeviceState attribute.
 */
	virtual bool is_DeviceState_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for PsStat attribute.
 */
	virtual bool is_PsStat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Waveform attribute.
 */
	virtual bool is_Waveform_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for On command.
 */
	virtual bool is_On_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Off command.
 */
	virtual bool is_Off_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Reset command.
 */
	virtual bool is_Reset_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCurrent command.
 */
	virtual bool is_GetCurrent_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetVoltage command.
 */
	virtual bool is_GetVoltage_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetVdcLink command.
 */
	virtual bool is_GetVdcLink_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for WaveStartSw command.
 */
	virtual bool is_WaveStartSw_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for StartCycling command.
 */
	virtual bool is_StartCycling_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Abort command.
 */
	virtual bool is_Abort_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCurrentSet command.
 */
	virtual bool is_GetCurrentSet_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for SaveConfig command.
 */
	virtual bool is_SaveConfig_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for StartDegaussing command.
 */
	virtual bool is_StartDegaussing_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCmdLog command.
 */
	virtual bool is_GetCmdLog_allowed(const CORBA::Any &any);
/**
 * 
 *	@exception DevFailed
 */
	void	on();
/**
 * 
 *	@exception DevFailed
 */
	void	off();
/**
 * 
 *	@exception DevFailed
 */
	void	reset();
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_current(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_voltage(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_vdc_link(const Tango::DevVarLongArray *);
/**
 * Start waveform by a software trigger.
 *	@exception DevFailed
 */
	void	wave_start_sw();
/**
 * Start cycling procedure
 *	@exception DevFailed
 */
	void	start_cycling();
/**
 * Abort a cycling or a waveform generation
 *	@exception DevFailed
 */
	void	abort();
/**
 * Get current set values
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_current_set(const Tango::DevVarLongArray *);
/**
 * Save ps mac address into property
 *	@exception DevFailed
 */
	void	save_config();
/**
 * Start degaussing procedure
 *	@exception DevFailed
 */
	void	start_degaussing();
/**
 * 
 *	@param	argin	number of last commands
 *	@return	report
 *	@exception DevFailed
 */
	Tango::DevVarStringArray	*get_cmd_log(Tango::DevLong);

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	
	
	friend class cyclethread;
	friend class offthread;
	
	a2605_shm_t *a2605;
	void set_DevStat(void);
	void configure_network(void); 	
	
	void string_explode(string, string, vector<string>*);
	int axtoi(char *hexStg); 
	
	void send_reset_string(char *);
	
	
	/* realtime buffer vvariables */
	int fd_buffer[A2605_STORE_VALUES];
	struct _buffer *buf[A2605_STORE_VALUES];
	bool init_buffers(void);
	void delete_buffers(void);
	long vsize[A2605_STORE_VALUES];
	bool PsStat[32];
	bool comm_ok;
	
	Tango::WAttribute *attr_set_current;	
	Tango::DevBoolean attr_b_value;
	Tango::DevShort attr_s_value;
	Tango::DevLong attr_l_value;
	
	long wave_ps[A2605_WAVE_LEN];
	double dwave_ps[A2605_WAVE_LEN];
	
	bool cycle_abort, cycle_run, off_abort, off_run;
	
	/* to reset dsp */
	struct sockaddr_in mysocket;
	long write_cmd(char *in, char *out, int insize);
	int sock_connect();
	void sock_flush();
	int sock;
	void reset_dsp(void);
	
	long wave_ps_size;
	
	void check_comm(void);
	cyclethread *cycleloop;
	offthread *offloop;
	
	/* udp global vars */
	int fd_socket;
	struct sockaddr_in host_address;
	struct sockaddr_in target_host_address;	/*the receiver's address*/
	struct hostent *host;
	struct sockaddr_in client_addr;
	socklen_t addr_len;
 	struct timeval last_sock_time, last_data_time;
	map<int, int> char_to_hex;

	/* udp com functions */
	int create_udp_socket(void);
	void send_udp_socket(char *buffer);
	void receive_udp_socket(char *buffer, int *bytes_read);
	int delete_udp_socket(void);
	
	/* communication validation */
	void check_set_response(char *cmd_resp);
	void check_get_response(char *get_cmd, char *get_resp);
	
	/* ps api */
	void get_udp_current(Tango::DevDouble *current);
	void get_udp_voltage(Tango::DevDouble *voltage);
	void get_udp_dclink(Tango::DevDouble *dclink);
	void get_udp_stat(long *stat);
	void set_udp_current(Tango::DevDouble current);
	void set_udp_onoff(bool on_flag);
	void set_udp_reset(void);
	bool udp_cycle_flag;
	bool udp_off_flag;	
	
	void config_polling_attr(void);
	bool polling_config;

	omni_mutex::omni_mutex *in_mutex;
	struct timeval curr_read_diff_time;
	bool diff_err;
	
	bool degauss_flag;
	
	string host_rw;
	long port_rw; 
	
protected :	
	//	Add your own data members here
	//-----------------------------------------
};

}	// namespace_ns

#endif	// _A2605_H
