//=============================================================================
//
// file :         A2605Class.h
//
// description :  Include for the A2605Class root class.
//                This class is represents the singleton class for
//                the A2605 device class.
//                It contains all properties and methods which the 
//                A2605 requires only once e.g. the commands.
//			
// project :      TANGO Device Server
//
// $Author: giulio $
//
// $Revision: 1.6 $
//
// $Log: A2605Class.h,v $
// Revision 1.6  2013-04-16 09:44:23  giulio
// Update rt-buffer api's, removed tango rpath, added DeviceState
//
// Revision 1.5  2012-04-19 09:28:22  giulio
// Added command logging buffer
//
// Revision 1.4  2012-01-02 12:55:29  giulio
// .Changed logic in local+rnm current control, add degauss, updated cycling procedure
//
// Revision 1.3  2011/01/17 13:30:20  giulio
// .Set and save ps mac address
//
// Revision 1.2  2010/08/26 13:11:24  giulio
// Fixed a bug in get_wave api
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================

#ifndef _A2605CLASS_H
#define _A2605CLASS_H

#include <tango.h>
#include <A2605.h>


namespace A2605_ns
{//=====================================
//	Define classes for attributes
//=====================================
class WaveformAttrib: public Tango::SpectrumAttr
{
public:
	WaveformAttrib():SpectrumAttr("Waveform", Tango::DEV_DOUBLE, Tango::READ_WRITE, 20000) {};
	~WaveformAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_Waveform(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_Waveform(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_Waveform_allowed(ty);}
};

class PsStatAttrib: public Tango::SpectrumAttr
{
public:
	PsStatAttrib():SpectrumAttr("PsStat", Tango::DEV_BOOLEAN, Tango::READ, 32) {};
	~PsStatAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_PsStat(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_PsStat_allowed(ty);}
};

class DeviceStateAttrib: public Tango::Attr
{
public:
	DeviceStateAttrib():Attr("DeviceState", Tango::DEV_USHORT, Tango::READ_WRITE) {};
	~DeviceStateAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_DeviceState(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_DeviceState(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_DeviceState_allowed(ty);}
};

class PsMacAddressAttrib: public Tango::Attr
{
public:
	PsMacAddressAttrib():Attr("PsMacAddress", Tango::DEV_STRING, Tango::READ_WRITE) {};
	~PsMacAddressAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_PsMacAddress(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_PsMacAddress(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_PsMacAddress_allowed(ty);}
};

class CommFrequencyAttrib: public Tango::Attr
{
public:
	CommFrequencyAttrib():Attr("CommFrequency", Tango::DEV_DOUBLE, Tango::READ) {};
	~CommFrequencyAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_CommFrequency(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_CommFrequency_allowed(ty);}
};

class CurrentSetAttrib: public Tango::Attr
{
public:
	CurrentSetAttrib():Attr("CurrentSet", Tango::DEV_DOUBLE, Tango::READ_WRITE) {};
	~CurrentSetAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_CurrentSet(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_CurrentSet(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_CurrentSet_allowed(ty);}
};

class BunchNumberAttrib: public Tango::Attr
{
public:
	BunchNumberAttrib():Attr("BunchNumber", Tango::DEV_LONG, Tango::READ) {};
	~BunchNumberAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_BunchNumber(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_BunchNumber_allowed(ty);}
};

class CurrentControlAttrib: public Tango::Attr
{
public:
	CurrentControlAttrib():Attr("CurrentControl", Tango::DEV_SHORT, Tango::READ_WRITE) {};
	~CurrentControlAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_CurrentControl(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_CurrentControl(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_CurrentControl_allowed(ty);}
};

class WaveTrigSwAttrib: public Tango::Attr
{
public:
	WaveTrigSwAttrib():Attr("WaveTrigSw", Tango::DEV_BOOLEAN, Tango::READ_WRITE) {};
	~WaveTrigSwAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveTrigSw(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveTrigSw(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveTrigSw_allowed(ty);}
};

class WaveTrigIrqAttrib: public Tango::Attr
{
public:
	WaveTrigIrqAttrib():Attr("WaveTrigIrq", Tango::DEV_BOOLEAN, Tango::READ_WRITE) {};
	~WaveTrigIrqAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveTrigIrq(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveTrigIrq(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveTrigIrq_allowed(ty);}
};

class WaveTrigBunchNumAttrib: public Tango::Attr
{
public:
	WaveTrigBunchNumAttrib():Attr("WaveTrigBunchNum", Tango::DEV_BOOLEAN, Tango::READ_WRITE) {};
	~WaveTrigBunchNumAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveTrigBunchNum(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveTrigBunchNum(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveTrigBunchNum_allowed(ty);}
};

class WaveStepFactorAttrib: public Tango::Attr
{
public:
	WaveStepFactorAttrib():Attr("WaveStepFactor", Tango::DEV_LONG, Tango::READ_WRITE) {};
	~WaveStepFactorAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveStepFactor(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveStepFactor(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveStepFactor_allowed(ty);}
};

class WaveLoopModeAttrib: public Tango::Attr
{
public:
	WaveLoopModeAttrib():Attr("WaveLoopMode", Tango::DEV_BOOLEAN, Tango::READ_WRITE) {};
	~WaveLoopModeAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveLoopMode(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveLoopMode(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveLoopMode_allowed(ty);}
};

class WaveCyclesAttrib: public Tango::Attr
{
public:
	WaveCyclesAttrib():Attr("WaveCycles", Tango::DEV_LONG, Tango::READ_WRITE) {};
	~WaveCyclesAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveCycles(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveCycles(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveCycles_allowed(ty);}
};

class WaveBunchNumStartAttrib: public Tango::Attr
{
public:
	WaveBunchNumStartAttrib():Attr("WaveBunchNumStart", Tango::DEV_LONG, Tango::READ_WRITE) {};
	~WaveBunchNumStartAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveBunchNumStart(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveBunchNumStart(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveBunchNumStart_allowed(ty);}
};

class WaveBunchNumEndAttrib: public Tango::Attr
{
public:
	WaveBunchNumEndAttrib():Attr("WaveBunchNumEnd", Tango::DEV_LONG, Tango::READ) {};
	~WaveBunchNumEndAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveBunchNumEnd(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveBunchNumEnd_allowed(ty);}
};

class WaveAbsCurModeAttrib: public Tango::Attr
{
public:
	WaveAbsCurModeAttrib():Attr("WaveAbsCurMode", Tango::DEV_BOOLEAN, Tango::READ_WRITE) {};
	~WaveAbsCurModeAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_WaveAbsCurMode(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_WaveAbsCurMode(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_WaveAbsCurMode_allowed(ty);}
};

class VoltageAttrib: public Tango::Attr
{
public:
	VoltageAttrib():Attr("Voltage", Tango::DEV_DOUBLE, Tango::READ) {};
	~VoltageAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_Voltage(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_Voltage_allowed(ty);}
};

class VdcLinkAttrib: public Tango::Attr
{
public:
	VdcLinkAttrib():Attr("VdcLink", Tango::DEV_DOUBLE, Tango::READ) {};
	~VdcLinkAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_VdcLink(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_VdcLink_allowed(ty);}
};

class CurrentFbAttrib: public Tango::Attr
{
public:
	CurrentFbAttrib():Attr("CurrentFb", Tango::DEV_DOUBLE, Tango::READ) {};
	~CurrentFbAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_CurrentFb(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_CurrentFb_allowed(ty);}
};

class CurrentAttrib: public Tango::Attr
{
public:
	CurrentAttrib():Attr("Current", Tango::DEV_DOUBLE, Tango::READ_WRITE) {};
	~CurrentAttrib() {};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<A2605 *>(dev))->read_Current(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
	{(static_cast<A2605 *>(dev))->write_Current(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<A2605 *>(dev))->is_Current_allowed(ty);}
};

//=========================================
//	Define classes for commands
//=========================================
class GetCmdLogCmd : public Tango::Command
{
public:
	GetCmdLogCmd(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetCmdLogCmd(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetCmdLogCmd() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_GetCmdLog_allowed(any);}
};



class StartDegaussingCmd : public Tango::Command
{
public:
	StartDegaussingCmd(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	StartDegaussingCmd(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~StartDegaussingCmd() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_StartDegaussing_allowed(any);}
};



class SaveConfigClass : public Tango::Command
{
public:
	SaveConfigClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	SaveConfigClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~SaveConfigClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_SaveConfig_allowed(any);}
};



class GetCurrentSetClass : public Tango::Command
{
public:
	GetCurrentSetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetCurrentSetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetCurrentSetClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_GetCurrentSet_allowed(any);}
};



class AbortClass : public Tango::Command
{
public:
	AbortClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	AbortClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~AbortClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_Abort_allowed(any);}
};



class StartCyclingClass : public Tango::Command
{
public:
	StartCyclingClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	StartCyclingClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~StartCyclingClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_StartCycling_allowed(any);}
};



class WaveStartSwClass : public Tango::Command
{
public:
	WaveStartSwClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	WaveStartSwClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~WaveStartSwClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_WaveStartSw_allowed(any);}
};



class GetVdcLinkClass : public Tango::Command
{
public:
	GetVdcLinkClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetVdcLinkClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetVdcLinkClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_GetVdcLink_allowed(any);}
};



class GetVoltageClass : public Tango::Command
{
public:
	GetVoltageClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetVoltageClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetVoltageClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_GetVoltage_allowed(any);}
};



class GetCurrentClass : public Tango::Command
{
public:
	GetCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetCurrentClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_GetCurrent_allowed(any);}
};



class ResetClass : public Tango::Command
{
public:
	ResetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	ResetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~ResetClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_Reset_allowed(any);}
};



class OffClass : public Tango::Command
{
public:
	OffClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	OffClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~OffClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_Off_allowed(any);}
};



class OnClass : public Tango::Command
{
public:
	OnClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	OnClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~OnClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<A2605 *>(dev))->is_On_allowed(any);}
};



//
// The A2605Class singleton definition
//

class
#ifdef WIN32
	__declspec(dllexport)
#endif
	A2605Class : public Tango::DeviceClass
{
public:
//	properties member data

//	add your own data members here
//------------------------------------

public:
	Tango::DbData	cl_prop;
	Tango::DbData	cl_def_prop;
	Tango::DbData	dev_def_prop;

//	Method prototypes
	static A2605Class *init(const char *);
	static A2605Class *instance();
	~A2605Class();
	Tango::DbDatum	get_class_property(string &);
	Tango::DbDatum	get_default_device_property(string &);
	Tango::DbDatum	get_default_class_property(string &);
	
protected:
	A2605Class(string &);
	static A2605Class *_instance;
	void command_factory();
	void get_class_property();
	void attribute_factory(vector<Tango::Attr *> &);
	void write_class_property();
	void set_default_property();

private:
	void device_factory(const Tango::DevVarStringArray *);
};


}	//	namespace A2605_ns

#endif // _A2605CLASS_H
