//
// file :         a2605thread.cpp
//
// description :  psa2605 acquisition thread
//
// project :      TANGO Device Server
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//

#include <omnithread.h>
#include "A2605.h"
#include "cyclethread.h"

namespace A2605_ns
{
/* 
 *	a2605thread::run()
 *	Run  
 */		
void cyclethread::run(void *) 
{

	int num_cycles = 3;
	long idx = myacq->psIndex;
	double min_cur = (double)myacq->a2605->ps[idx].iset_min_rd/1000000,
	       max_cur = (double)myacq->a2605->ps[idx].iset_max_rd/1000000;
	int sleep_time;
	double cur_array[1000];
	long size_cur_array = (long)(max_cur-min_cur)*2+1;
	
	if (myacq->realtimeControl) {
		sleep_time = myacq->a2605->ps[myacq->psIndex].tx_period*2;   /* usec */	
	}
	else {
		sleep_time = 1000000;
	}
		
	myacq->cycle_run = true;
	
	if (myacq->realtimeControl) {
		myacq->a2605->ps[myacq->psIndex].stat_rd |= A2605_RD_STAT_CYCLE;
	}
	else
		myacq->udp_cycle_flag = true;
	
	if (myacq->degauss_flag == true)
		num_cycles = 6;
	
	
	/* keep sleep time between ranges */
	if (sleep_time > 1000000)
		sleep_time = 999999;
	if (sleep_time < 200000)
		sleep_time = 150000;
	
	try {
	
	/* start from 0 */
	if (myacq->realtimeControl)
		a2605_set_current(idx, 0, myacq->a2605);
	else
		myacq->set_udp_current(0);
	sleep(1);
	omni_thread::sleep(1,0);
	
	double tmp_max_cur = -1000, tmp_min_cur = 1000;
	long tmp_idx_max_cur = 0, tmp_idx_min_cur = 0;
	for (int i = 0; i <= size_cur_array; i++) {
		cur_array[i] = sin(2*3.14159265*i/size_cur_array)*max_cur;
		if (cur_array[i] > tmp_max_cur) {
			tmp_max_cur = cur_array[i];
			tmp_idx_max_cur = i;
		}
		if (cur_array[i] < tmp_min_cur) {
			tmp_min_cur = cur_array[i];
			tmp_idx_min_cur = i;
		}		
	}
	cur_array[tmp_idx_max_cur] = max_cur;
	cur_array[tmp_idx_min_cur] = -max_cur;

	
	double min_degauss_cur = 0.1;
	double red_cur = 0;
	while (1) {
		if ((max_cur/min_degauss_cur) > pow(red_cur+0,num_cycles-1))
			red_cur+=0.1;
		else
			break;	 
	}
	
	for (int i = 0; i < num_cycles; i++) {
	
		if ((myacq->degauss_flag == true) && (i > 0)){
			for (int j = 0; j <= size_cur_array; j++) {
				cur_array[j] /= red_cur;
			}
		}

		for (int j = 0; j <= size_cur_array; j++) {

			if (myacq->cycle_abort)
				break;			
			if (myacq->realtimeControl)
				a2605_set_current(idx, (long)(cur_array[j]*1000000), myacq->a2605); /* from A to uA */
			else {
				myacq->set_udp_current(cur_array[j]);
				myacq->attr_Current_write = (cur_array[j]);
			}
			if ((cur_array[j] == max_cur) || (cur_array[j] == -max_cur))
				omni_thread::sleep(0,sleep_time*3000);
			else
				omni_thread::sleep(0,sleep_time*1000);
		}
			
		if (myacq->cycle_abort)
			break;
			
	} /* for i */

	} /* try */ 
	catch(Tango::DevFailed e) {
		
	}

	if (myacq->realtimeControl) {
		myacq->a2605->ps[myacq->psIndex].stat_rd &= ~A2605_RD_STAT_CYCLE;
	}
	else
		myacq->udp_cycle_flag = false;
	

	try {
	/* start from 0 */
	if (myacq->realtimeControl)
		a2605_set_current(idx, 0, myacq->a2605);
	else {
		myacq->set_udp_current(0);	
		myacq->attr_Current_write = 0;
	}
	} /* try */
	catch(Tango::DevFailed e) {
	}

	myacq->cycle_abort = false;
	myacq->cycle_run = false;	
	myacq->degauss_flag = false;
		
}





}
